@@ -16,32 +16,17 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-class Rational;
-class Track;
-
 class Bitmap : public Rectangle
   {
   std::vector< std::vector< char > > data;	// char is faster than bool
 
 public:
-  struct Error
-    {
-    const char * s;
-    Error( const char * p ) { s = p; }
-    };
-
-  // Creates a Bitmap from a pbm, pgm or ppm file
-  Bitmap( FILE * f, const Rational & th, const bool invert ) throw( Error );
-
-  // Creates a blank Bitmap
+      // Creates a blank Bitmap
   Bitmap( const int l, const int t, const int r, const int b ) throw();
 
-  // Creates a Bitmap from part of another Bitmap
+      // Creates a Bitmap from part of another Bitmap
   Bitmap( const Bitmap & source, const Rectangle & re ) throw();
 
-  // Creates a reduced bitmap
-  Bitmap( const Bitmap & source, const int scale, const Rational & th ) throw();
-
   using Rectangle::left;
   using Rectangle::top;
   using Rectangle::right;
@@ -80,13 +65,4 @@ public:
   int  follow_bottom( int row, int col ) const throw();
   bool top_hook   ( int *hdiff ) const throw();
   bool bottom_hook( int *hdiff ) const throw();
-
-  int  analyse_layout( std::vector< Rectangle > & rv,
-                       const int layout_level ) const throw();
-  void draw_rectangle( const Rectangle & re ) throw();
-  void draw_track( const Track & tr ) throw();
-  void histogramize( const bool vertical = true ) throw();
-  void save( FILE * f, char filetype = '4' ) const throw();
-  bool scale( int n, const Rational & th ) throw( Error );
-  void transform( const Transformation & t ) throw();
   };
