@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,18 +50,8 @@ public:
   //  id = 1 for block dots, negative for hole dots, 0 otherwise
   int id( const int row, const int col ) const throw();
 
-  bool escape_left  ( int row, int col ) const throw();
-  bool escape_top   ( int row, int col ) const throw();
-  bool escape_right ( int row, int col ) const throw();
-  bool escape_bottom( int row, int col ) const throw();
-  int  follow_top   ( int row, int col ) const throw();
-  int  follow_bottom( int row, int col ) const throw();
-  bool top_hook   ( int *hdiff ) const throw();
-  bool bottom_hook( int *hdiff ) const throw();
-
   void print( FILE * outfile ) const throw();
 
-  bool adjust_height() throw();
   void fill_hole( const int i ) throw();
   void find_holes() throw();
   };
