@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
 #include "ucs.h"
@@ -74,6 +75,16 @@ Block & Character::block( int i ) throw()
   }
 
 
+Block & Character::main_block() throw()
+  {
+  int imax = 0;
+  for( int i = 1; i < blocks(); ++i )
+    if( bpv[i]->size() > bpv[imax]->size() )
+      imax = i;
+  return *bpv[imax];
+  }
+
+
 void Character::shift_blockp( Block * p ) throw()
   {
   add_rectangle( *p );
@@ -138,6 +149,22 @@ bool Character::maybe( int code ) const throw()
   return false;
   }
 
+/*
+bool Character::maybe_digit() const throw()
+  {
+  for( int i = 0; i < guesses(); ++i )
+    if( UCS::isdigit( gv[i].code ) ) return true;
+  return false;
+  }
+
+
+bool Character::maybe_letter() const throw()
+  {
+  for( int i = 0; i < guesses(); ++i )
+    if( UCS::isalpha( gv[i].code ) ) return true;
+  return false;
+  }
+*/
 
 void Character::join( Character & c ) throw()
   {
@@ -158,7 +185,8 @@ void Character::print( const Control & control ) const throw()
         if( ch ) std::putc( ch, control.outfile ); }
         break;
       case Control::utf8:
-        std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
+        if( gv[0].code )
+          std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
       }
     }
   else std::putc( '_', control.outfile );
@@ -273,3 +301,43 @@ void Character::xprint( const Control & control ) const throw()
       }
   std::fputs( "\n", control.exportfile );
   }
+
+
+void Character::apply_filter( const Filter & filter ) throw()
+  {
+  if( filter.type() == Filter::none ) return;
+  const int code = guesses() ? gv[0].code : 0;
+  bool remove = false;
+
+  switch( filter.type() )
+    {
+    case Filter::none:			// only for completeness
+      break;
+    case Filter::letters_only:
+      remove = true;
+    case Filter::letters:
+      if( !UCS::isalpha( code ) && !UCS::isspace( code ) )
+        {
+        for( int i = 1; i < guesses(); ++i )
+          if( UCS::isalpha( gv[i].code ) ) { swap_guesses( 0, i ); break; }
+        if( guesses() && !UCS::isalpha( gv[0].code ) )
+          gv[0].code = UCS::to_nearest_letter( gv[0].code );
+        if( remove && ( !guesses() || !UCS::isalpha( gv[0].code ) ) )
+          only_guess( 0, 0 );
+        }
+      break;
+    case Filter::numbers_only:
+      remove = true;
+    case Filter::numbers:
+      if( !UCS::isdigit( code ) && !UCS::isspace( code ) )
+        {
+        for( int i = 1; i < guesses(); ++i )
+          if( UCS::isdigit( gv[i].code ) ) { swap_guesses( 0, i ); break; }
+        if( guesses() && !UCS::isdigit( gv[0].code ) )
+          gv[0].code = UCS::to_nearest_digit( gv[0].code );
+        if( remove && ( !guesses() || !UCS::isdigit( gv[0].code ) ) )
+          only_guess( 0, 0 );
+        }
+      break;
+    }
+  }
