@@ -76,7 +76,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
   code = f.test_G();                 if( code ) { add_guess( code, 0 ); return; }
   code = f.test_c();                 if( code ) { add_guess( code, 0 ); return; }
   if( charset.enabled( Charset::iso_8859_9 ) )
-    { code = f.test_s_cedilla(); if( code ) { add_guess( code, 0 ); return; } }
+    { code = f.test_s_cedilla();     if( code ) { add_guess( code, 0 ); return; } }
   code = f.test_235Esz( charset );   if( code ) { add_guess( code, 0 ); return; }
   code = f.test_HKMNUuvwYy( charbox );
   if( code == 'u' && f.lp.istpit() )	// Looks for merged 'tr'
@@ -95,7 +95,8 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
         }
       }
     }
-  if( code == 'N' && b.width() > b.height() && b.top() > charbox.top() )
+  if( code == 'N' && b.width() > b.height() && b.top() >= charbox.top() &&
+      4 * f.tp[f.tp.pos(50)] < b.height() )
     {					// Looks for merged 'rv'
     const int col = f.hp.iminimum();
     if( col > f.hp.pos( 40 ) && col < f.hp.pos( 50 ) )
@@ -105,9 +106,12 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
       }
     }
   if( code ) { add_guess( code, 0 ); return; }
-  if( f.bp.minima( b.height() / 8 ) == 2 )
+  const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
+  if( f.bp.minima() <= 2 &&
+      ( f.bp.minima( b.height() / 8 + noise ) == 2 ||
+        ( b.height() >= 16 && f.bp.minima( b.height() / 8 ) == 2 ) ) )
     {
-    code = f.test_hknwx();
+    code = f.test_hknwx( charbox );
     if( code == 'n' )	// Looks for '"' or merged 'rr' or 'TT' or 'rt' or 'fl'
       {
       if( b.bottom() <= charbox.vcenter() ) { add_guess( '"', 0 ); return; }
@@ -234,7 +238,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
   code = f.test_line( charbox );
   if( code ) { add_guess( code, 0 ); return; }
 
-  code = f.test_misc();
+  code = f.test_misc( charbox );
   if( code ) { add_guess( code, 0 ); return; }
   }
 
@@ -325,6 +329,13 @@ void Character::recognize112( const Rectangle & charbox ) throw()
   const int a1 = h1.area();
   const int a2 = h2.area();
 
+  {
+  const int w = b.right() - std::min( b.hcenter(),
+                            std::min( h1.hcenter(), h2.hcenter() ) );
+  for( int i = h1.bottom() - b.top() + 1; i < h2.top() - b.top(); ++i )
+    if( rp[i] > w ) { add_guess( 'g', 2 ); return; }
+  }
+
   if( Ocrad::similar( a1, a2, 50 ) )		// I don't like this
     {
     if( h1.bottom() > b.vcenter() && h2.top() < b.vcenter() &&
@@ -337,9 +348,6 @@ void Character::recognize112( const Rectangle & charbox ) throw()
         if( b.top_hook( &hdiff ) && hdiff > b.height() / 2 )
           { add_guess( 's', 0 ); return; }
     
-      for( int i = h1.bottom() - b.top() + 1; i < h2.top() - b.top(); ++i )
-        if( 2 * rp[i] > b.width() ) { add_guess( 'g', 2 ); return; }
-
       if( lp.isflats() && !lp.ispit() ) { add_guess( 'B', 0 ); return; }
 
       int col1 = h1.seek_left( h1.bottom(), h1.right() + 1 ) - 1;
@@ -383,7 +391,7 @@ void Character::recognize112( const Rectangle & charbox ) throw()
         }
 
       if( charbox.vcenter() > h1.top() &&
-          ( charbox.vcenter() <= h1.bottom() || charbox.bottom() < h2.vcenter() ) )
+          ( charbox.vcenter() < h1.bottom() || charbox.bottom() < h2.vcenter() ) )
         add_guess( 'g', 2 );
       add_guess( 'B', 1 ); add_guess( 'a', 0 );
       return;
