@@ -41,6 +41,21 @@ public:
   };
 
 
+class Filter
+  {
+public:
+  enum Type { none, letters, letters_only, numbers, numbers_only };
+private:
+  Type _type;
+
+public:
+  Filter() throw() : _type( none ) {}
+  bool set( const char * name ) throw();
+  Type type() const throw() { return _type; }
+  void show_error( const char * program_name, const char * arg ) const throw();
+  };
+
+
 class Transformation
   {
 public:
@@ -62,6 +77,7 @@ struct Control
   enum Format { byte, utf8 };
 
   Charset charset;
+  Filter filter;
   Format format;
   FILE *outfile, *exportfile;
   int debug_level;
