@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
 #include "ucs.h"
@@ -41,7 +42,8 @@ int Features::hbars() const throw()
   if( _hbars < 0 )
     {
     const Block & b = *_block;
-    int state = 0, begin = 0, l = 0, r = 0, limit = wp.max() / 2;
+    const int limit = wp.max() / 2;
+    int state = 0, begin = 0, l = 0, r = 0;
     std::vector< int > count( b.height(), 0 );
     _hbars = 0;
 
@@ -69,12 +71,13 @@ int Features::hbars() const throw()
                   }
                 state = 0;
                 int end = ( maxcount <= limit ) ? row - 1 : row;
-                int width = r - l + 1;
+                const int width = r - l + 1;
                 while( begin <= end && 3 * count[begin-b.top()] < 2 * width )
                   ++begin;
                 while( begin <= end && 3 * count[end-b.top()] < 2 * width )
                   --end;
-                if( begin > end ) break;
+                const int height = end - begin + 1;
+                if( height < 1 || 2 * height > 3 * width ) break;
                 _hbar.push_back( Rectangle( l, begin, r, end ) );
                 ++_hbars; break;
         }
@@ -152,7 +155,7 @@ const std::vector< int > & Features::hscan() const throw()
         bool black = b.get_bit( row, col );
         switch( state )
           {
-          case 0: if( black ) state = 1; break;
+          case 0: if( black ) state = 1; else break;
           case 1: if( !black || row == b.bottom() )
                     { state = 0; ++_hscan[col-b.left()]; }
           }
@@ -181,7 +184,7 @@ const std::vector< int > & Features::vscan() const throw()
         bool black = b.get_bit( row, col );
         switch( state )
           {
-          case 0: if( black ) state = 1; break;
+          case 0: if( black ) state = 1; else break;
           case 1: if( !black || col == b.right() )
                     { state = 0; ++_vscan[row-b.top()]; }
           }
@@ -192,7 +195,7 @@ const std::vector< int > & Features::vscan() const throw()
   }
 
 
-int Features::test_misc() const throw()
+int Features::test_misc( const Rectangle & charbox ) const throw()
   {
   const Block & b = *_block;
 
@@ -207,24 +210,12 @@ int Features::test_misc() const throw()
       b.seek_left( b.vcenter(), b.hcenter() ) <= b.left() )
     return '7';
 
-  if( b.height() > 2 * b.width() && rp.increasing( 1 ) && tp.decreasing( 1 ) &&
-      lp.iscpit( 25 ) )
-    return '1';
-
   if( tp.minima( b.height() / 4 ) == 1 && bp.minima( b.height() / 4 ) == 1 )
     {
-    if( b.height() > 2 * wp.max() )
-      {
-      if( hbars() == 1 && hbar(0).bottom() >= b.bottom() - 1 ) return 'l';
-      if( hbars() == 2 && hbar(0).bottom() < b.vpos( 25 ) &&
-          hbar(1).bottom() >= b.bottom() - 1 )
-        {
-        if( hbar(0).right() <= hbar(1).hcenter() ) return 0;
-        if( 3 * hbar(0).width() <= 2 * hbar(1).width() ||
-            b.height() > 3 * wp.max() ) return 'l';
-        return 'I';
-        }
-      }
+    if( b.height() > 2 * b.width() && rp.increasing( 1 ) &&
+        tp.decreasing( 1 ) && lp.iscpit( 25 ) )
+      return '1';
+
     if( hbars() == 1 ||
         ( hbars() == 2 && hbar(1).bottom() >= b.bottom() - 1 &&
         3 * hbar(0).width() > 4 * hbar(1).width() ) )
@@ -234,21 +225,56 @@ int Features::test_misc() const throw()
         if( 3 * wp[i] < b.width() && 5 * lp[i] > b.width() &&
             5 * rp[i] > b.width() ) return 'T';
         }
+
+    if( 3 * b.height() > 4 * b.width() &&
+        vbars() == 1 && vbar(0).width() >= 2 )
+      {
+      const int lg = vbar(0).left() - b.left();
+      const int rg = b.right() - vbar(0).right();
+      if( 2 * lg < b.width() && 2 * rg < b.width() &&
+          Ocrad::similar( lg, rg, 40 ) &&
+          4 * bp[bp.pos(25)] > 3 * b.height() &&
+          4 * tp[tp.pos(75)] > 3 * b.height() )
+        return 'l';
+      }
+
+    if( 5 * b.height() >= 4 * charbox.height() && b.height() > wp.max() &&
+        3 * wp[wp.pos(50)] < b.width() )
+      {
+      if( hbars() == 1 && hbar(0).bottom() >= b.bottom() - 1 &&
+          hbar(0).top() > b.vpos( 75 ) )
+        return 'l';
+      if( hbars() == 2 && hbar(0).bottom() < b.vpos( 25 ) &&
+          hbar(1).top() > b.vpos( 75 ) &&
+          hbar(1).bottom() >= b.bottom() - 1 /*&&
+          3 * hbar(0).width() < 4 * hbar(1).width()*/ )
+        {
+        if( hbar(0).right() <= hbar(1).hcenter() ) return 0;
+        if( 3 * hbar(0).width() <= 2 * hbar(1).width() ||
+            b.height() > 3 * wp.max() ) return 'l';
+        return 'I';
+        }
+      }
+
     if( ( hbars() == 2 || hbars() == 3 ) && hbar(0).top() <= b.top() + 1 &&
         hbar(1).includes_vcenter( b ) &&
         3 * hbar(0).width() > 4 * hbar(1).width() &&
         ( hbars() == 2 ||
           ( hbar(2).bottom() >= b.bottom() - 1 &&
             3 * hbar(0).width() > 4 * hbar(2).width() ) ) ) return 'F';
-    }
 
-  if( b.height() > 3 * wp.max() )
-    {
-    if( rp.istip() && lp.ispit() )
-      { if( lp.istpit() ) return '{'; else return '('; }
-    if( lp.istip() && rp.ispit() )
-      { if( rp.istpit() ) return '}'; else return ')'; }
-    if( b.width() > 2 * wp.max() && rp.isconvex() ) return ')';
+    if( b.height() > 3 * wp.max() )
+      {
+      if( rp.istip() && lp.ispit() )
+        { if( lp.istpit() ) return '{'; else return '('; }
+      if( lp.istip() && rp.ispit() )
+        { if( rp.istpit() ) return '}'; else return ')'; }
+      if( b.width() > 2 * wp.max() && rp.isconvex() ) return ')';
+      }
+
+    if( b.height() > 2 * b.width() && 5 * b.height() >= 4 * charbox.height() &&
+        lp.max() + rp.max() < b.width() )
+      return '|';
     }
 
   return 0;
