@@ -48,12 +48,12 @@ public:
   int test_frst( const Rectangle & charbox ) const throw();
   int test_G() const throw();
   int test_HKMNUuvwYy( const Rectangle & charbox ) const throw();
-  int test_hknwx() const throw();
+  int test_hknwx( const Rectangle & charbox ) const throw();
   int test_s_cedilla() const throw();
 
   bool test_comma() const throw();
   int test_easy( const Rectangle & charbox ) const throw();
   int test_line( const Rectangle & charbox ) const throw();
   int test_solid( const Rectangle & charbox ) const throw();
-  int test_misc() const throw();
+  int test_misc( const Rectangle & charbox ) const throw();
   };
