@@ -171,8 +171,9 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
     }
 
   const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
-  const int topmax = b.top() + noise;
-  const int botmin = b.bottom() - noise;
+  const int vnoise = ( b.height() / 30 ) + 1;
+  const int topmax = b.top() + vnoise;
+  const int botmin = b.bottom() - vnoise;
   if( vbars() == 1 && vbar(0).width() >= 2 && 2 * vbar(0).width() < b.width() )
     {
     if( std::abs( vbar(0).hcenter() - b.hcenter() ) <= noise &&
@@ -184,8 +185,9 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
         if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
             Ocrad::similar( b.height(), b.width(), 50 ) ) return '+';
         }
-      if( hbars() == 2 && 4 * hbar(0).height() <= b.height() &&
-          hbar(0).top() <= topmax && hbar(1).bottom() >= botmin &&
+      if( hbars() == 2 &&
+          hbar(0).top() <= topmax && 4 * hbar(0).height() <= b.height() &&
+          hbar(1).bottom() >= botmin && 4 * hbar(1).height() <= b.height() &&
           3 * hbar(0).width() > 4 * hbar(1).width() )
         return 'T';
       }
@@ -196,33 +198,30 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
     {
     if( vbar(0).right() <= b.hcenter() )
       {
-      if( hbars() == 3 && hbar(0).top() <= topmax &&
+      if( ( hbars() == 2 || hbars() == 3 ) && hbar(0).top() <= topmax &&
           hbar(0).width() + 1 >= hbar(1).width() &&
           2 * hbar(1).width() >= 3 * vbar(0).width() &&
           vbar(0).h_overlaps( hbar(1) ) )
         {
-        if( Ocrad::similar( hbar(0).width(), hbar(2).width(), 10 ) &&
+        if( hbars() == 3 &&
+            Ocrad::similar( hbar(0).width(), hbar(2).width(), 10 ) &&
             10 * hbar(2).width() >= 9 * hbar(1).width() &&
             hbar(0).left() <= hbar(1).left() + 1 )
           return 'E';
-        if( hbar(0).width() > hbar(2).width() &&
-            hbar(1).includes_vcenter( b ) )
+        if( ( hbars() == 2 || hbar(0).width() > hbar(2).width() ) &&
+            ( hbar(1).includes_vcenter( b ) ||
+              ( 3 * hbar(1).width() > 2 * hbar(0).width() &&
+                10 * lp[vnoise] < b.width() && hbar(1).top() > b.vpos( 30 ) &&
+                hbar(1).bottom() < b.vpos( 60 ) ) ) )
           return 'F';
         }
-      if( hbars() == 2 )
-        {
-        if( hbar(0).top() <= topmax &&
-            hbar(0).width() + 1 >= hbar(1).width() &&
-            2 * hbar(1).width() >= 3 * vbar(0).width() &&
-            hbar(1).includes_vcenter( b ) )
-          return 'F';
-        if( hbar(1).bottom() >= botmin && b.height() > b.width() &&
-            hbar(1).width() > hbar(0).width() &&
-            std::abs( vbar(0).hcenter() - hbar(0).hcenter() ) <= 1 &&
-            rp.iminimum() > rp.pos( 70 ) )
-          return 'L';
-        }
-      if( hbars() == 1 && Ocrad::similar( hbar(0).width(), b.width(), 10 ) )
+      if( hbars() == 2 && hbar(1).bottom() >= botmin &&
+          b.height() > b.width() && hbar(1).width() > hbar(0).width() &&
+          std::abs( vbar(0).hcenter() - hbar(0).hcenter() ) <= 1 &&
+          rp.iminimum() > rp.pos( 70 ) )
+        return 'L';
+      if( hbars() == 1 && Ocrad::similar( hbar(0).width(), b.width(), 10 ) &&
+          vbar(0).left() <= b.hpos( 30 ) )
         {
         if( hbar(0).bottom() >= botmin &&
             b.escape_top( b.vcenter(), b.hpos( 75 ) ) )
@@ -250,9 +249,9 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
     {
     if( 3 * b.height() > 4 * b.width() &&
         Ocrad::similar( vbar(0).left() - b.left(),
-                        b.right() - vbar(0).right(), 30, 2 ) )
+                        b.right() - vbar(0).right(), 30, 2 * noise ) )
       {
-      if( rp.istip() && lp.istip() && b.height() <= 3 * wp.max() )
+      if( b.height() <= 3 * wp.max() && rp.istip() && lp.istip() )
         {
         if( b.height() <= 3 * b.width() &&
             lp[lp.pos(40)] > lp[lp.pos(60)] + noise &&
@@ -261,7 +260,8 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
         }
       if( rp.isflats() &&
           ( lp.istip() || lp.isflats() ||
-            ( lp.isctip() && lp.minima() == 2 && lp.iminimum(0) < lp.pos( 30 ) && lp.iminimum(1) > lp.pos( 80 ) ) ) )
+            ( lp.isctip() && lp.minima() == 2 &&
+              lp.iminimum() < lp.pos( 30 ) && lp.iminimum(1) > lp.pos( 80 ) ) ) )
         return 'l';
       if( b.height() > 3 * wp.max() )
         {
@@ -358,6 +358,7 @@ int Features::test_frst( const Rectangle & charbox ) const throw()
 
   if( bp.minima( b.height() / 4 ) != 1 || tp.minima( b.height() / 2 ) != 1 ||
       bp.minima( b.height() / 2 ) != 1 ) return 0;
+  const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
   const bool maybe_slanted_r = ( tp.minima( b.height() / 4 ) != 1 );
   bool maybe_t = true;
 
@@ -432,15 +433,19 @@ int Features::test_frst( const Rectangle & charbox ) const throw()
       {
       const int col = b.right() - std::max( 0, rp[rp.pos(50)] - 1 );
       if( !b.escape_bottom( b.vcenter(), col ) )
-        { if( 3 * wp[wp.pos(6)] < 2 * b.width() && tp.ispit() && lp.iminimum() < lp.pos( 40 ) ) return 't'; }
+        {
+        if( 3 * wp[wp.pos(6)] < 2 * b.width() && tp.ispit() &&
+            lp.iminimum() < lp.pos( 40 ) ) return 't';
+        else return 0;
+        }
       else if( 2 * wp.max() > b.width() )
         {
         if( rp.iminimum() < rp.pos( 20 ) )
           {
           if( rp.increasing( rp.pos( 20 ) ) || bp.increasing() ||
-              tp.minima( 1 ) == 2 ||
+              tp.minima( noise ) == 2 ||
               ( rp.minima() == 1 && ( b.height() < charbox.height() || tp.iminimum() > tp.pos( 50 ) ) ) )
-            return 'r';
+            { if( b.height() <= 3 * wp.max() ) return 'r'; else return 0; }
           else if( 3 * b.height() >= 5 * b.width() ) return 'f';
           }
         else
@@ -469,7 +474,7 @@ int Features::test_G() const throw()
   if( lp.isconvex() || lp.ispit() )
     {
     int col = 0, row = 0;
-    for( int i = rp.pos( 40 ); i <= rp.pos( 60 ); ++i )
+    for( int i = rp.pos( 30 ); i <= rp.pos( 60 ); ++i )
       if( rp[i] > col ) { col = rp[i]; row = i; }
     if( col == 0 ) return 0;
     row += b.top(); col = b.right() - col + 1;
@@ -508,6 +513,7 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
   if( tp.minima( b.height() / 5 ) == 2 && tp.minima( b.height() / 4 ) == 2 &&
       tp.isctip() )
     {
+    const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
     if( 2 * b.height() >= b.width() && b.height() >= 10 &&
         bp.minima( b.height() / 5 ) == 1 )
       {
@@ -536,7 +542,7 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
       if( bp.ispit() && 7 * tp.range() < 4 * b.height() &&
           2 * wp[wp.pos(70)] < b.width() &&
           ( ( Ocrad::similar( lg, rg, 20 ) &&
-            5 * wp[wp.pos(98)] >= 4 * wp[wp.pos(70)] ) ||
+            5 * wp[wp.pos(98)] + noise >= 4 * wp[wp.pos(70)] ) ||
             ( lg > 1 && lg < rg && lc >= lc2 && !rp.increasing() ) ) ) return 'Y';
       if( b.escape_top( b.vpos( 60 ), b.hcenter() ) && !lp.istip() )
         return 'u';
@@ -553,7 +559,6 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
     if( 2 * b.height() >= b.width() && b.height() >= 10 &&
         bp.minima() == 2 && bp.isctip() )
       {
-      const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
       const int th = std::max( b.height() / 4, bp[bp.pos(50)] + noise );
       if( bp.minima( th ) == 3 ) return 'M';
       const int lg = lp[lp.pos(50)];
@@ -564,10 +569,17 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
         if( lg > 1 && rg > 1 && lp.increasing() && rp.increasing() )
           return 'w';
         if( hbars() == 1 && 5 * hbar(0).height() < b.height() &&
-            hbar(0).v_includes( b.vcenter() ) &&
-            10 * hbar(0).h_distance( b.hcenter() ) <= b.width() )
-          return 'H';
-        if( vscan()[rp.pos(60)] == 4 || vscan()[rp.pos(70)] == 4 ) return 'w';
+            hbar(0).top() >= b.vpos( 30 ) && hbar(0).bottom() <= b.vpos( 60 ) &&
+            10 * hbar(0).width() > 9 * wp[hbar(0).vcenter()-b.top()] )
+          {
+          if( 9 * hbar(0).width() < 10 * wp[wp.pos(50)] ) return 'H';
+          return 0;
+          }
+        if( vscan()[rp.pos(60)] == 4 || vscan()[rp.pos(70)] == 4 )
+          {
+          if( 2 * tp[tp.pos(50)] > b.height() ) return 'M';
+          return 'w';
+          }
         if( ( vbars() <= 2 || ( vbars() == 3 && b.height() >= b.width() ) ) &&
             tp.minima( b.height() / 2 ) <= 2 &&
             tp.minima( ( 2 * b.height() ) / 5 ) <= 2 && !lp.istpit() &&
@@ -596,14 +608,19 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
 // Looks for the nearest frontier in column hcenter(), then tests if
 // gap is open downwards (except for 'x')
 //
-int Features::test_hknwx() const throw()
+int Features::test_hknwx( const Rectangle & charbox ) const throw()
   {
   const Block & b = *_block;
+  const int m8 = tp.minima( b.height() / 8 );
 
-  if( tp.minima( b.height() / 8 ) == 2 && bp.minima( b.height() / 2 ) == 1 &&
+  if( m8 == 2 && bp.minima( b.height() / 2 ) == 1 &&
       ( ( lp.istip() && rp.istip() ) ||
         ( lp.isconcave() && rp.isconcave() ) ) ) return 'x';
 
+  if( b.width() >= b.height() && tp.ispit() &&
+      ( b.bottom() < charbox.vcenter() || ( lp.decreasing() && rp.decreasing() ) ) )
+    return '^';
+
   int col = 0, row = 0;
   for( int i = bp.pos( 40 ); i <= bp.pos( 60 ); ++i )
     if( bp[i] > row ) { row = bp[i]; col = i; }
@@ -613,16 +630,15 @@ int Features::test_hknwx() const throw()
   { int c = col; col = b.seek_right( row, col ); if( col > c ) --col;
   row = b.seek_top( row, col ); }
 
-  if( tp.ispit() && b.width() >= b.height() && lp.decreasing() && rp.decreasing() )
-    return '^';
-  int urow = b.seek_top( row - 1, col, false );
-  if( urow > b.vpos( 20 ) )
+  const int urow = b.seek_top( row - 1, col, false );
+  if( urow > b.vpos( 20 ) || 3 * tp[tp.pos(60)] > b.height() )
     {
     const int m5 = tp.minima( b.height() / 5 );
     if( ( m5 == 2 || m5 == 3 ) && tp.minima() >= 2 &&
         rp[rp.pos(25)] <= b.width() / 4 &&
         ( !lp.istpit() || rp.minima() == 1 ) ) return 'w';
-    if( m5 == 1 ) { if( rp.isctip( 66 ) ) return 'k'; else return 'h'; }
+    if( m5 == 1 && m8 == 1 && 4 * tp.max( tp.pos(40), tp.pos(60) ) < 3 * b.height() )
+      { if( rp.isctip( 66 ) ) return 'k'; else return 'h'; }
     return 0;
     }
   if( Ocrad::similar( b.height(), b.width(), 40 ) && row > b.vcenter() &&
@@ -741,7 +757,11 @@ int Features::test_line( const Rectangle & charbox ) const throw()
     if( slope1 > 0 && slope2 < 0 )
       {
       if( b.bottom() > charbox.vcenter() )
-        { if( 2 * b.width() >= b.height() ) return '\\'; else return 0; }
+        {
+        if( ( 3 * b.width() > b.height() && b.height() > charbox.height() ) ||
+            2 * b.width() >= b.height() ) return '\\';
+        else return 0;
+        }
       return '`';
       }
     return 0;
@@ -750,22 +770,26 @@ int Features::test_line( const Rectangle & charbox ) const throw()
   if( bp.minima() == 1 && 2 * b.width() >= b.height() &&
       tp.straight( &slope1 ) && bp.straight( &slope2 ) )
     {
-    if( slope1 < 0 && slope2 < 0 && lp.minima() == 1 && rp.minima() == 2 )
-      return '<';
-    if( slope1 > 0 && slope2 > 0 && lp.minima() == 2 && rp.minima() == 1 )
-      return '>';
-    if( lp.minima() != 1 || rp.minima() != 1 ) return 0;
-    if( slope1 < 0 && slope2 > 0 )
+    if( lp.minima() == 1 && rp.minima() == 1 )
       {
-      if( b.v_includes( charbox.vcenter() ) ) return '/';
-      if( b.top() >= charbox.vcenter() ) return ',';
-      return '\'';
+      if( slope1 < 0 && slope2 > 0 )
+        {
+        if( b.v_includes( charbox.vcenter() ) ) return '/';
+        if( b.top() >= charbox.vcenter() ) return ',';
+        return '\'';
+        }
+      if( slope1 > 0 && slope2 < 0 )
+        {
+        if( b.bottom() > charbox.vcenter() ) return '\\';
+        return '`';
+        }
       }
-    if( slope1 > 0 && slope2 < 0 )
+    else if( 2 * b.width() >= b.height() )
       {
-      if( b.bottom() > charbox.vcenter() )
-        { if( 2 * b.width() >= b.height() ) return '\\'; else return 0; }
-      return '`';
+      if( slope1 < 0 && slope2 < 0 && lp.minima() == 1 && rp.minima() == 2 )
+        return '<';
+      if( slope1 > 0 && slope2 > 0 && lp.minima() == 2 && rp.minima() == 1 )
+        return '>';
       }
     }
   return 0;
@@ -804,7 +828,7 @@ int Features::test_solid( const Rectangle & charbox ) const throw()
 
   if( Ocrad::similar( b.height(), wp.max(), 20, 2 ) )
     {
-    int n = std::min( b.height(), b.width() );
+    const int n = std::min( b.height(), b.width() );
     if( n >= 6 )
       {
       int d = 0;
@@ -825,8 +849,9 @@ int Features::test_solid( const Rectangle & charbox ) const throw()
       ( porosity && inner_area * 100 < inner_size * 95 ) ) return 0;
   if( b.width() > b.height() )
     {
-    if( b.width() >= 6 * b.height() &&
-        charbox.bottom() - b.bottom() < b.top() - charbox.vcenter() ) return '_';
+    if( b.top() > charbox.vpos( 90 ) ||
+        ( charbox.bottom() - b.bottom() < b.top() - charbox.vcenter() &&
+          b.width() >= 5 * b.height() ) ) return '_';
     return '-';
     }
 
