@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
 #include "bitmap.h"
@@ -312,31 +313,31 @@ bool Profile::ispit() throw()
     {
     _ispit = false; if( _limit < 0 ) initialize();
     if( samples() < 5 ) return _ispit;
-    const int nsize = samples() / 30;
-    for( int i = 0; i <= nsize; ++i )
-      if( data[i] < nsize + 2 - i || data[samples()-i-1] < nsize + 2 - i )
+    const int noise = ( std::min( samples(), _limit ) / 25 ) + 1;
+    for( int i = 0; i < noise; ++i )
+      if( data[i] <= noise - i || data[samples()-i-1] <= noise - i )
         return _ispit;
 
-    const int dmin = min();
+    const int dmin = min(), dmax = _limit / 2;
     int begin = 0, end = 0, i, ref;
-    for( i = 0, ref = _limit; i < samples(); ++i )
+    for( i = 0, ref = dmax; i < samples(); ++i )
       {
       int d = data[i];
       if( d == dmin ) { begin = i; break; }
-      if( d < ref ) ref = d; else if( d > ref + 1 ) return _ispit;
+      if( d < ref ) ref = d; else if( d > ref + noise && ref < dmax ) return _ispit;
       }
     if( begin < 2 || begin > samples() - 3 ) return _ispit;
 
-    for( i = samples() - 1, ref = _limit; i >= begin; --i )
+    for( i = samples() - 1, ref = dmax; i >= begin; --i )
       {
       int d = data[i];
       if( d == dmin ) { end = i; break; }
-      if( d < ref ) ref = d; else if( d > ref + 1 ) return _ispit;
+      if( d < ref ) ref = d; else if( d > ref + noise && ref < dmax ) return _ispit;
       }
     if( end < begin || end > samples() - 3 ) return _ispit;
 
     for( i = begin + 1; i < end; ++i )
-      if( data[i] > dmin + 1 ) return _ispit;
+      if( data[i] > dmin + noise ) return _ispit;
 
     _ispit = true;
     }
@@ -372,8 +373,8 @@ bool Profile::islpit() throw()
   {
   if( _limit < 0 ) initialize();
   if( samples() < 5 ) return false;
-  const int nsize = samples() / 30;
-  if( data[0] < nsize + 2 ) return false;
+  const int noise = samples() / 30;
+  if( data[0] < noise + 2 ) return false;
 
   const int dmin = min();
   int begin = 0, ref = _limit;
@@ -554,12 +555,13 @@ int Profile::imaximum() throw()
 int Profile::iminimum( int m, int th ) throw()
   {
   if( _limit < 0 ) initialize();
-  if( samples() < 1 ) return false;
+  const int margin = ( samples() / 30 ) + 1;
+  if( samples() < 2 * margin ) return 0;
   if( th < 2 ) th = ( mean() < 2 ) ? 2 : mean();
   int minima = 0, status = 0;
   int begin = 0, end, value = _limit + 1;
 
-  for( end = 0; end < samples(); ++end )
+  for( end = margin; end < samples() - margin; ++end )
     {
     if( status == 0 )
       { if( data[end] < th ) { status = 1; ++minima; begin = end; } }
@@ -579,7 +581,7 @@ int Profile::iminimum( int m, int th ) throw()
 int Profile::minima( int th ) throw()
   {
   if( _limit < 0 ) initialize();
-  if( !samples() ) return false;
+  if( !samples() ) return 0;
   if( th < 1 ) th = ( mean() < 2 ) ? 2 : mean();
   const int noise = _limit / 40;
   const int dth = th - ( ( noise + 1 ) / 2 ), uth = th + ( noise / 2 );
