@@ -16,6 +16,7 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
+#include <algorithm>
 #include <cstdio>
 #include <vector>
 
@@ -67,11 +68,14 @@ void join_characters( std::vector< Textline * > & tlpv ) throw()
         if( c1.vcenter() < c2.vcenter() ) cup = &c1, cdn = &c2;
         else cup = &c2, cdn = &c1;
         if( cdn->includes_hcenter( *cup ) || cup->includes_hcenter( *cdn ) ||
-            ( cdn->top() > cup->bottom() && cdn->hcenter() < cup->hcenter() ) )
+            ( cdn->top() > cup->bottom() && cdn->hcenter() < cup->hcenter() ) ||
+            ( cdn->blocks() == 2 &&
+              2 * cdn->block( 0 ).size() < cdn->block( 1 ).size() &&
+              cdn->block( 0 ).includes_vcenter( *cup ) ) )
           {
           int k;
-          if( 64 * c1.size() < c2.size() ) k = i;
-          else if( 64 * c2.size() < c1.size() ) k = j;
+          if( 64 * c1.size() < c2.main_block().size() ) k = i;
+          else if( 64 * c2.size() < c1.main_block().size() ) k = j;
           else if( cdn == &c2 ) { c2.join( c1 ); k = i; }
           else { c1.join( c2 ); k = j; }
           line.delete_character( k );
@@ -152,8 +156,8 @@ void insert_spaces( std::vector< Textline * > & tlpv ) throw()
 } // end namespace
 
 
-Textblock::Textblock( const Rectangle & r, const Charset & charset,
-           std::vector< std::vector< Block * > > & blockp_matrix ) throw()
+Textblock::Textblock( const Rectangle & r,
+                      std::vector< std::vector< Block * > > & blockp_matrix ) throw()
   : Rectangle( r )
   {
   int cuts = blockp_matrix.size();
@@ -319,6 +323,23 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
       }
     }
 
+  // remove clipped lines at top or bottom of text block
+  if( textlines() )
+    {
+    const Textline & l = *tlpv[textlines()-1];
+    for( int i = 0, c = 0; i < l.characters(); ++i )
+      if( l.character( i ).bottom() >= bottom() && 2 * ++c >= l.characters() )
+        { delete_line( tlpv, textlines() - 1 ); break; }
+    }
+  if( textlines() )
+    {
+    const Textline & l = *tlpv[0];
+    const int t = std::max( top(), 1 );
+    for( int i = 0, c = 0; i < l.characters(); ++i )
+      if( l.character( i ).top() <= t && 2 * ++c >= l.characters() )
+        { delete_line( tlpv, 0 ); break; }
+    }
+
   // Second pass. Join lines of i-dots and tildes.
   for( int current_line = 0; current_line < textlines() - 1; )
     {
@@ -330,7 +351,7 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
       for( int i1 = 0; !joined && i1 < line1.characters(); ++i1 )
         {
         Character & c1 = line1.character( i1 );
-        if( c1.height() * 2 >= line2.mean_height() ) continue;
+        if( 2 * c1.height() >= line2.mean_height() ) continue;
         for( int i2 = 0; !joined && i2 < line2.characters(); ++i2 )
           {
           Character & c2 = line2.character( i2 );
@@ -390,9 +411,18 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
         l.delete_character( l.characters() - 1 );
       }
     }
-
   insert_spaces( tlpv );
+  }
 
+
+Textblock::~Textblock() throw()
+  {
+  for( int i = textlines() - 1; i >= 0; --i ) delete tlpv[i];
+  }
+
+
+void Textblock::recognize( const Charset & charset, const Filter & filter ) throw()
+  {
   // Recognize characters.
   for( int i = 0; i < textlines(); ++i )
     {
@@ -402,6 +432,10 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
     tlpv[i]->recognize2( charset );
     }
 
+  if( filter.type() != Filter::none )
+    for( int i = 0; i < textlines(); ++i )
+      tlpv[i]->apply_filter( filter );
+
   // Remove unrecognized lines.
   for( int i = textlines() - 1; i >= 0; --i )
     {
@@ -417,27 +451,33 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
   // Add blank lines.
   if( textlines() >= 3 )
     {
-    int mean_vdistance = ( tlpv.back()->mean_vcenter() - tlpv.front()->mean_vcenter() ) / ( textlines() - 1 );
-    if( mean_vdistance > 0 )
-      for( int current_line = 0; current_line + 1 < textlines(); ++current_line )
+    int min_vdistance = ( tlpv.back()->mean_vcenter() - tlpv.front()->mean_vcenter() ) / ( textlines() - 1 );
+    for( int i = 0; i + 1 < textlines(); ++i )
+      {
+      const Textline & line1 = *tlpv[i];
+      const Textline & line2 = *tlpv[i+1];
+      if( !Ocrad::similar( line1.characters(), line2.characters(), 50 ) ||
+          !Ocrad::similar( line1.width(), line2.width(), 30 ) ) continue;
+      const int vdistance = line2.mean_vcenter() - line1.mean_vcenter();
+      if( vdistance >= min_vdistance ) continue;
+      const int mh1 = line1.mean_height(), mh2 = line2.mean_height();
+      if( mh1 < 10 || mh2 < 10 ) continue;
+      if( Ocrad::similar( mh1, mh2, 20 ) && 2 * vdistance > mh1 + mh2 )
+        min_vdistance = vdistance;
+      }
+    if( min_vdistance > 0 )
+      for( int i = 0; i + 1 < textlines(); ++i )
         {
-        Textline & line1 = *tlpv[current_line];
-        Textline & line2 = *tlpv[current_line+1];
-        int vdistance = line2.mean_vcenter() - line1.mean_vcenter();
-        if( vdistance <= 0 ) continue;
-        int newlines = (vdistance + (mean_vdistance / 2)) / mean_vdistance;
-        for( int i = 1; i < newlines; ++i ) insert_line( tlpv, ++current_line );
+        const Textline & line1 = *tlpv[i];
+        const Textline & line2 = *tlpv[i+1];
+        int vdistance = line2.mean_vcenter() - line1.mean_vcenter() - min_vdistance;
+        while( 2 * vdistance > min_vdistance )
+          { insert_line( tlpv, ++i ); vdistance -= min_vdistance; }
         }
     }
   }
 
 
-Textblock::~Textblock() throw()
-  {
-  for( int i = textlines() - 1; i >= 0; --i ) delete tlpv[i];
-  }
-
-
 int Textblock::characters() const throw()
   {
   int total = 0;
