@@ -21,9 +21,10 @@ class Textblock : public Rectangle
   mutable std::vector< Textline * > tlpv;
 
 public:
-  Textblock( const Rectangle & r, const Charset & charset,
+  Textblock( const Rectangle & r,
              std::vector< std::vector< Block * > > & blockp_matrix ) throw();
   ~Textblock() throw();
+  void recognize( const Charset & charset, const Filter & filter ) throw();
 
 //  Textline & textline( int i ) const throw();
   int textlines() const throw() { return tlpv.size(); }
