@@ -41,8 +41,10 @@ public:
   void join( Textline & tl ) throw();
 
   Character & character( int i ) const throw();
-  Rectangle charbox( const Character & c ) const throw();
   int characters() const throw() { return cpv.size(); }
+  Rectangle charbox( const Character & c ) const throw();
+  int width() const throw()
+    { return cpv.size() ? cpv.back()->right() - cpv[0]->left() : 0; }
 
   int mean_height() const throw();
   Rational mean_width() const throw();
@@ -57,4 +59,5 @@ public:
 
   void recognize1( const Charset & charset ) const throw();
   void recognize2( const Charset & charset ) throw();
+  void apply_filter( const Filter & filter ) throw();
   };
