@@ -251,8 +251,8 @@ void Textline::recognize2( const Charset & charset ) throw()
           {
           int code2 = c2.guess(0).code;
           if( code2 >= 128 ) continue;
-          if( ( std::isalpha( code2 ) && 5 * c1.height() < 4 * c2.height() ) ||
-              ( std::islower( code2 ) && !UCS::islower_small_ambiguous( code2 ) &&
+          if( ( std::isupper( code2 ) && code2 != 'B' && code2 != 'Q' && 5 * c1.height() < 4 * c2.height() ) ||
+              ( UCS::islower_small( code2 ) && code2 != 'r' && !UCS::islower_small_ambiguous( code2 ) &&
                 ( c1.height() <= c2.height() ||
                   Ocrad::similar( c1.height(), c2.height(), 10 ) ) ) )
             { c1.swap_guesses( 0, 1 ); break; }
@@ -290,10 +290,19 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform some small letters to capitals
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  {
+  int begin = 0;
+  bool isolated = false;	// isolated letters compare with all line
+  for( int i = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
-    if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
+    if( c1.maybe(' ') )
+      {
+      if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
+        { begin = 0; isolated = true; }
+      else { begin = i + 1; isolated = false; }
+      continue;
+      }
     if( c1.guesses() == 1 )
       {
       int code = c1.guess( 0 ).code;
@@ -304,7 +313,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         {
         Character & c2 = character( j );
         if( !c2.guesses() ) continue;
-        if( c2.maybe(' ') ) break;
+        if( c2.maybe(' ') ) { if( isolated ) continue; else break; }
         int code2 = c2.guess( 0 ).code;
         if( code2 >= 128 || !std::isalpha( code2 ) ) continue;
         if( !capital )
@@ -315,10 +324,10 @@ void Textline::recognize2( const Charset & charset ) throw()
                      Ocrad::similar( c1.height(), c2.height(), 10 ) ) )
             capital = true;
           }
-        if( !small && std::islower( code2 ) && code2 != 'j' )
+        if( !small && std::islower( code2 ) && code2 != 'l' && code2 != 'j' )
           {
           if( 5 * c1.height() < 4 * c2.height() ) small = true;
-          else if( UCS::islower_small( code2 ) &&
+          else if( UCS::islower_small( code2 ) && code2 != 'r' &&
                    ( j < i || !UCS::islower_small_ambiguous( code2 ) ) &&
                    Ocrad::similar( c1.height(), c2.height(), 10 ) )
             small = true;
@@ -327,6 +336,7 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( capital && !small ) c1.insert_guess( 0, std::toupper( code ), 1 );
       }
     }
+  }
 
   // transform 'i' into 'j'
   for( int i = 0; i < characters(); ++i )
@@ -345,12 +355,21 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform small o or u with accent or diaeresis to capital
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  {
+  int begin = 0;
+  bool isolated = false;	// isolated letters compare with all line
+  for( int i = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() >= 1 )
       {
-      if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
+      if( c1.maybe(' ') )
+        {
+        if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
+          { begin = 0; isolated = true; }
+        else { begin = i + 1; isolated = false; }
+        continue;
+        }
       int code = c1.guess( 0 ).code;
       if( code < 128 || c1.blocks() < 2 ) continue;
       int codeb = UCS::base_letter( code );
@@ -361,7 +380,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         Character & c2 = character( j );
         if( c2.guesses() >= 1 )
           {
-          if( c2.maybe(' ') ) break;
+          if( c2.maybe(' ') ) { if( isolated ) continue; else break; }
           int code2 = c2.guess( 0 ).code;
           int code2b = UCS::base_letter( code2 );
           if( !code2b && code2 >= 128 ) continue;
@@ -374,6 +393,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         }
       }
     }
+  }
 
   // transform 'O' or 'l' into '0' or '1'
   for( int i = 0, begin = 0; i < characters(); ++i )
@@ -391,7 +411,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         if( c2.guesses() >= 1 )
           {
           int code2 = c2.guess( 0 ).code;
-          if( code2 < 128 && std::isdigit( code2 ) )
+          if( UCS::isdigit( code2 ) )
             {
             if( Ocrad::similar( c1.height(), c2.height(), 10 ) )
               c1.insert_guess( 0, (code == 'l') ? '1' : '0', c1.guess(0).value + 1 );
@@ -415,7 +435,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         {
         Character & c2 = character( i + 1 );
         int code = c2.guess( 0 ).code;
-        if( code < 128 && ( std::isalnum( code ) || code == '.' || code == '|' ) )
+        if( UCS::isalnum( code ) || code == '.' || code == '|' )
           {
           valid_c2 = true;
           switch( code )
@@ -442,8 +462,7 @@ void Textline::recognize2( const Charset & charset ) throw()
       Character & c2 = character( i );
       if( !c2.guesses() ) continue;
       int code = c2.guess( 0 ).code;
-      if( UCS::isvowel( code ) || ( code < 128 && std::isalnum( code ) ) ||
-          code == '.' || code == '|' )
+      if( UCS::isalnum( code ) || code == '.' || code == '|' )
         {
         switch( code )
           {
@@ -494,7 +513,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     if( !c1.guesses() ) continue;
     int height = c1.height();
     int code = c1.guess( 0 ).code;
-    if( std::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' )
+    if( UCS::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' )
       {
       int digits = 1;
       int i = begin + 1;
@@ -504,7 +523,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         if( !c.guesses() ) break;
         bool valid = false;
         int code = c.guess(0).code;
-        if( ( std::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' ) &&
+        if( ( UCS::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' ) &&
             Ocrad::similar( c.height(), height, 10 ) )
           { valid = true; ++digits; }
         if( code == '.' || code == ',' || code == ':' || code == '+' || code == '-' )
@@ -570,10 +589,10 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( rcode == '|' && ( !lcode || !UCS::isalnum( lcode ) ) )
         {
         if( i < characters() - 2 && character( i + 2 ).guesses() &&
-            std::isalpha( character( i + 2 ).guess( 0 ).code ) )
+            UCS::isalpha( character( i + 2 ).guess( 0 ).code ) )
           { c.insert_guess( 0, 'l', 1 ); continue; }
         if( i >= 2 && character( i - 2 ).guesses() &&
-            std::isalpha( character( i - 2 ).guess( 0 ).code ) )
+            UCS::isalpha( character( i - 2 ).guess( 0 ).code ) )
           { c.insert_guess( 0, 'l', 1 ); continue; }
         }
       }
@@ -666,7 +685,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         else if( i < characters() - 1 && character( i + 1 ).guesses() )
           {
           int code1 = character( i + 1 ).guess( 0 ).code;
-          if( ( std::isdigit( code1 ) || UCS::isupper( code1 ) || code1 == 'i' ) &&
+          if( ( UCS::isdigit( code1 ) || UCS::isupper( code1 ) || code1 == 'i' ) &&
               5 * c.height() > 4 * character( i + 1 ).height() )
           c.swap_guesses( 0, 1 );
           }
