@@ -344,55 +344,58 @@ Textpage::Textpage( const Page_image & page_image,
                     const char * filename, const Control & control ) throw()
   : Rectangle( page_image ), name( filename )
   {
-  if( control.debug_level < 0 || control.debug_level > 100 ) return;
+  const int debug_level = control.debug_level;
+  if( debug_level < 0 || debug_level > 100 ) return;
 
   std::vector< Zone > zone_vector;
-  scan_page( page_image, zone_vector, control.debug_level );
+  scan_page( page_image, zone_vector, debug_level );
 
-  if( control.debug_level >= 98 )
+  if( debug_level >= 98 )
     {
     if( control.outfile ) bprint( zone_vector, control.outfile );
     return;
     }
-  if( control.debug_level > 89 ) return;
+  if( debug_level > 95 || ( debug_level > 89 && debug_level < 94 ) ) return;
 
   // build a Textblock for every zone with text
   for( unsigned int i = 0; i < zone_vector.size(); ++i )
     {
     Textblock * tbp = new Textblock( zone_vector[i].rectangle,
-                      control.charset, zone_vector[i].blockp_matrix );
+                                     zone_vector[i].blockp_matrix );
+    if( debug_level < 90 )
+      tbp->recognize( control.charset, control.filter );
     if( tbp->textlines() ) tbpv.push_back( tbp );
     else delete tbp;
     }
-  if( control.debug_level == 0 )
+  if( debug_level == 0 )
     {
     if( control.outfile ) print( control );
     if( control.exportfile ) xprint( control );
     return;
     }
   if( !control.outfile ) return;
-  if( control.debug_level >= 86 )
+  if( debug_level >= 86 )
     {
-    bool graph = ( control.debug_level >= 88 );
-    bool recursive = ( control.debug_level & 1 );
+    bool graph = ( debug_level >= 88 );
+    bool recursive = ( debug_level & 1 );
     for( int i = 0; i < textblocks(); ++i )
       tbpv[i]->dprint( control, graph, recursive );
     return;
     }
-  if( control.debug_level > 77 ) return;
-  if( control.debug_level >= 70 )
+  if( debug_level > 77 ) return;
+  if( debug_level >= 70 )
     {
     Page_image tmp( page_image );
-    if( ( control.debug_level - 70 ) & 1 )	// mark zones
+    if( ( debug_level - 70 ) & 1 )	// mark zones
       {
       for( unsigned int i = 0; i < zone_vector.size(); ++i )
         tmp.draw_rectangle( zone_vector[i].rectangle );
       }
-    if( ( control.debug_level - 70 ) & 2 )	// mark lines
+    if( ( debug_level - 70 ) & 2 )	// mark lines
       {
       for( int i = 0; i < textblocks(); ++i ) tbpv[i]->lmark( tmp );
       }
-    if( ( control.debug_level - 70 ) & 4 )	// mark characters
+    if( ( debug_level - 70 ) & 4 )	// mark characters
       {
       for( int i = 0; i < textblocks(); ++i ) tbpv[i]->cmark( tmp );
       }
